import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { AppDispatch, RootState } from './store';
import { GraphQueryResultFromBackend, GraphStatistics, NodeAttributes, QueryStatusResult } from 'ts-common';

export type UniqueEdge = {
  attributes: NodeAttributes;
  source: string;
  target: string;
  count: number;
};

// Define a type for the slice state
export type GraphQueryResult = GraphQueryResultFromBackend & {
  queryingBackend: boolean;
  metaData?: GraphStatistics;
};

// Define the initial state using that type
export const initialState: GraphQueryResult = {
  metaData: undefined,
  nodes: [],
  edges: [],
  queryingBackend: false,
};

export const graphQueryResultSlice = createSlice({
  name: 'graphQueryResult',
  initialState,
  reducers: {
    setNewGraphQueryResult: (state, action: PayloadAction<QueryStatusResult>) => {
      // Assign new state
      state.metaData = action.payload.result.payload.metaData;
      state.nodes = action.payload.result.payload.nodes;
      state.edges = action.payload.result.payload.edges;
      state.queryingBackend = false;
    },
    resetGraphQueryResults: state => {
      // Assign new state
      state.metaData = undefined;
      state.nodes = [];
      state.edges = [];
      state.queryingBackend = false;
    },
    queryingBackend: state => {
      state.queryingBackend = true;
    },
  },
});

export const { resetGraphQueryResults, queryingBackend } = graphQueryResultSlice.actions;

/**
 * Asynchronously sets a new graph query result.
 * Prefer to use this then the synchronous version since if the result is very large,
 *  the entire website will freeze while graphQueryBackend2graphQuery runs.
 *
 * This function dispatches an action to update the graph query result
 * in the store with the provided payload.
 *
 * @param payload - The result of the graph query to be set.
 * @param dispatcher - The dispatch function to trigger the action.
 */
export const asyncSetNewGraphQueryResult = async (payload: QueryStatusResult, dispatcher: AppDispatch) => {
  dispatcher(graphQueryResultSlice.actions.setNewGraphQueryResult(payload));
};

// Other code such as selectors can use the imported `RootState` type
export const selectGraphQueryResult = (state: RootState) => state.graphQueryResult;
export const selectGraphQueryResultNodes = (state: RootState) => state.graphQueryResult.nodes;
export const selectGraphQueryResultLinks = (state: RootState) => state.graphQueryResult.edges;
export const selectGraphQueryResultMetaData = (state: RootState) => state.graphQueryResult.metaData;

export default graphQueryResultSlice.reducer;
